norm.reg.mcmc <- function(y,X,beta.mn,beta.var,s2.mn,s2.sd,n.mcmc,no.print=FALSE){

#
#  Code Box 11.1
#

###
### Subroutines 
###

library(mvtnorm)

invgammastrt <- function(igmn,igvar){
  q <- 2+(igmn^2)/igvar
  r <- 1/(igmn*(q-1))
  list(r=r,q=q)
}

invgammamnvr <- function(r,q){  #  This fcn is not necessary
  mn <- 1/(r*(q-1))
  vr <- 1/((r^2)*((q-1)^2)*(q-2))
  list(mn=mn,vr=vr)
}

###
### Hyperpriors
###

n=dim(X)[1]
p=dim(X)[2]
n.burn=round(.1*n.mcmc)
r=invgammastrt(s2.mn,s2.sd^2)$r
q=invgammastrt(s2.mn,s2.sd^2)$q
Sig.beta.inv=diag(p)/beta.var

beta.save=matrix(0,p,n.mcmc)
s2.save=rep(0,n.mcmc)
Dbar.save=rep(0,n.mcmc)
y.pred.mn=rep(0,n)

###
### Starting Values
###

beta=solve(t(X)%*%X)%*%t(X)%*%y

###
### MCMC Loop
###

for(k in 1:n.mcmc){
  if(k%%100==0) cat(k," ")

  ###
  ### Sample s2
  ###

  tmp.r=(1/r+.5*t(y-X%*%beta)%*%(y-X%*%beta))^(-1)
  tmp.q=n/2+q

  s2=1/rgamma(1,tmp.q,,tmp.r) 

  ###
  ### Sample beta
  ###

  tmp.var=solve(t(X)%*%X/s2 + Sig.beta.inv)
  tmp.mn=tmp.var%*%(t(X)%*%y/s2 + Sig.beta.inv%*%beta.mn)

  beta=as.vector(rmvnorm(1,tmp.mn,tmp.var,method="chol"))

  ###
  ### Save Samples
  ###
  
  beta.save[,k]=beta
  s2.save[k]=s2

}
cat("\n")

###
###  Write Output
###

list(beta.save=beta.save,s2.save=s2.save,y=y,X=X,n.mcmc=n.mcmc,n=n,r=r,q=q,p=p)

}
